#ifndef __MSG_QUEUE_H_INCLUDED
#define __MSG_QUEUE_H_INCLUDED

#include <pthread.h>
#include "wintype.h"
#include "msg.h"

/* internal structure of a message queue */
typedef struct tagMSG_QUEUE {
  struct tagMSG_QUEUE *next;    /* allow us to be in a singly-linked list */
  PMSG startbound;              /* start boundary for message buffer */
  PMSG endbound;                /* end boundary for message buffer */
  PMSG head;                    /* head pointer of message queue */
  PMSG tail;                    /* tail pointer of message queue */
  UINT32 nentries;              /* number of entries possible in message buffer */
  pthread_mutex_t mutex;        /* controls access to queue from multiple threads */
  MSG messagestore[0];          /* message buffer */
} MSG_QUEUE, *PMSG_QUEUE;

/* flags to Mq_peek */
#define PEEK_REMOVE   0x0001    /* remove message if found */
#define PEEK_NOREMOVE 0x0000    /* do not remove message if found */

extern PMSG_QUEUE Mq_alloc(UINT32 nentries);
extern void Mq_destroy(PMSG_QUEUE queue);
extern void Mq_post(PMSG_QUEUE queue, HANDLE target, UINT32 message, const UINT_PTR *attrs, int nattrs);
extern void Mq_post2(PMSG_QUEUE queue, HANDLE target, UINT32 message, UINT_PTR attr1, UINT_PTR attr2);
extern BOOL Mq_peek(PMSG_QUEUE queue, PMSG msg, UINT32 flags);
extern BOOL Mq_is_empty(PMSG_QUEUE queue);

#define Mq_post0(q, tgt, msg)         Mq_post2(q, tgt, msg, 0, 0)
#define Mq_post1(q, tgt, msg, attr)   Mq_post2(q, tgt, msg, attr, 0)

#endif /* __MSG_QUEUE_H_INCLUDED */
